BRCA preliminary analysis
================

-   [BRCA](#brca)
-   [Using L1, L2, Elastic net and each combination with DegreeCox](#using-l1-l2-elastic-net-and-each-combination-with-degreecox)
    -   [General Options](#general-options)
    -   [Load libraries](#load-libraries)
        -   [Setup parallel computing if necessary](#setup-parallel-computing-if-necessary)
    -   [Data](#data)
        -   [Prepare survival data](#prepare-survival-data)
        -   [Description of the data](#description-of-the-data)
    -   [General functions to prepare data](#general-functions-to-prepare-data)
        -   [Select data with expression levels](#select-data-with-expression-levels)

BRCA
====

Using L1, L2, Elastic net and each combination with DegreeCox
=============================================================

General Options
---------------

``` r
nfolds       <- 20   # number of folds in cross-validation
nlambda      <- 100  # number of lambdas that should be calculated
is.parallel  <- TRUE # use parallel computing (if available) in cross-validation?
#
# if all samples should be used, then training set = test set, otherwise
#  the two sets are complements.
training.set  <- .85
training.seed <- 65087 # reproducible results
#
```

Load libraries
--------------

### Setup parallel computing if necessary

``` r
# Parallel
registerDoMC(cores=14) 
```

Data
----

loading data from package and filter only by CLL

``` r
# load rnaseq data
data(tissue)
data(clinical)

my.data <- list()
xdata       <- cbind(tissue$primary.solid.tumor) #, tissue$metastatic)
rownames(xdata) <- rownames(gdc$rnaseq)

my.data$clinical <- rbind(clinical$primary.solid.tumor) #, clinical$metastatic)
my.data$clinical <- my.data$clinical[unique(my.data$clinical$bcr_patient_barcode),]

# check if all patients in ge are in clinical
ge.codes <- getParticipantCode(colnames(xdata))
cl.codes <- my.data$clinical$bcr_patient_barcode

flog.info('Cases with GE but no clinical information:\n(if there are, they will be removed)', ge.codes[(!ge.codes %in% cl.codes)], capture = T)
```

    ## INFO [2016-12-23 16:10:22 29681] Cases with GE but no clinical information:
    ## (if there are, they will be removed)
    ## 
    ## [1] "TCGA-BH-A0B2"

``` r
xdata <- xdata[,(ge.codes %in% cl.codes)]
my.data$clinical <- my.data$clinical[getParticipantCode(colnames(xdata)),]

## remove where sd = 0
to.rm <- c()
for(ix in 1:nrow(xdata)) {
  if (sd(xdata[ix,]) == 0) {
    to.rm <- c(to.rm, ix)
  }
}
xdata <- xdata[-to.rm,]
flog.info('%d Variables removed as have standard deviation ==0', length(to.rm))
```

    ## INFO [2016-12-23 16:10:44 29681] 1569 Variables removed as have standard deviation ==0

``` r
# cleanup
rm(tissue)
rm(clinical)
rm(ge.codes)
rm(cl.codes)
```

### Prepare survival data

``` r
my.data$survComplete <- my.data$clinical[, c('days_to_death', "days_to_last_followup", "days_to_last_known_alive", "vital_status")]
#
vital.status.ix <- !is.na(my.data$survComplete$days_to_death)
flog.info('How follow-up dates when vital status == Dead? R: %s',
          sum(!is.na(my.data$survComplete[vital.status.ix,'days_to_last_followup'])))
```

    ## INFO [2016-12-23 16:10:44 29681] How follow-up dates when vital status == Dead? R: 0

``` r
ydata <- as.data.frame(cbind(array(0, nrow(my.data$survComplete)), my.data$survComplete[,'vital_status'] == 'Dead'))
colnames(ydata) <- c('time', 'status')
rownames(ydata) <- rownames(my.data$survComplete)

ydata$status <- factor(ydata$status, labels = c('ALIVE', 'DEAD'))

ydata$time[ vital.status.ix] <- my.data$survComplete[ vital.status.ix,'days_to_death']
ydata$time[!vital.status.ix] <- my.data$survComplete[!vital.status.ix,'days_to_last_followup']

flog.info('How any vital_status == NA? R: %d', sum(is.na(ydata$status)))
```

    ## INFO [2016-12-23 16:10:44 29681] How any vital_status == NA? R: 0

``` r
flog.info('How any days_to_{death,followup} == NA? R: %d', sum(is.na(ydata$time)))
```

    ## INFO [2016-12-23 16:10:44 29681] How any days_to_{death,followup} == NA? R: 1

``` r
to.rm <-rownames(ydata[is.na(ydata$time),])
ydata <- ydata[!rownames(ydata) %in% to.rm,]
xdata <- xdata[,!getParticipantCode(colnames(xdata)) %in% to.rm]
my.data$clinical <- my.data$clinical[!rownames(my.data$clinical) %in% to.rm,]

to.rm <- rownames(ydata[ydata$time <= 0,])
flog.info('How many with time <= 0?\n(if any they will be removed)', to.rm, capture = T)
```

    ## INFO [2016-12-23 16:10:46 29681] How many with time <= 0?
    ## (if any they will be removed)
    ## 
    ##  [1] "TCGA-C8-A1HO" "TCGA-C8-A12V" "TCGA-A8-A081" "TCGA-A8-A09G"
    ##  [5] "TCGA-E9-A1N5" "TCGA-A8-A083" "TCGA-E9-A1NH" "TCGA-C8-A12T"
    ##  [9] "TCGA-A8-A094" "TCGA-E9-A1N6" "TCGA-E9-A1NI" "TCGA-C8-A12K"
    ## [13] "TCGA-E9-A1N3" "TCGA-C8-A12O" "TCGA-E9-A229" "TCGA-E9-A1NC"
    ## [17] "TCGA-A8-A08H" "TCGA-E9-A22H" "TCGA-C8-A12Y" "TCGA-C8-A12U"
    ## [21] "TCGA-E9-A1NF" "TCGA-C8-A12M" "TCGA-A8-A09Z" "TCGA-C8-A133"
    ## [25] "TCGA-E9-A228" "TCGA-C8-A1HG" "TCGA-A8-A096" "TCGA-C8-A1HE"
    ## [29] "TCGA-C8-A1HF" "TCGA-C8-A1HI" "TCGA-PL-A8LV" "TCGA-E9-A22B"
    ## [33] "TCGA-E9-A1NE" "TCGA-E9-A1NG" "TCGA-E9-A1N9" "TCGA-A8-A090"
    ## [37] "TCGA-E9-A5UP" "TCGA-E9-A1ND" "TCGA-E9-A1NA" "TCGA-BH-A18H"
    ## [41] "TCGA-A8-A06N" "TCGA-C8-A12X" "TCGA-E9-A1R0" "TCGA-E9-A22G"
    ## [45] "TCGA-E9-A1QZ" "TCGA-E9-A22D" "TCGA-E9-A54X" "TCGA-E9-A22A"
    ## [49] "TCGA-C8-A12P" "TCGA-E9-A22E" "TCGA-C8-A12N" "TCGA-C8-A12L"
    ## [53] "TCGA-E9-A1N8"

``` r
ydata <- ydata[!rownames(ydata) %in% to.rm,]
xdata <- xdata[,!getParticipantCode(colnames(xdata)) %in% to.rm]

# cleanup
my.data$survComplete <- NULL
rm(vital.status.ix)
rm(to.rm)
```

### Description of the data

    ## INFO [2016-12-23 16:10:49 29681] Description of the data

    ## INFO [2016-12-23 16:10:49 29681]     Number of cases: 1047

    ## INFO [2016-12-23 16:10:49 29681] Number of variables: 55682

    ## INFO [2016-12-23 16:10:49 29681]  Clinical variables: 113

<img src="analysis_files/figure-markdown_github/description_data-1.png" width="750px" /><img src="analysis_files/figure-markdown_github/description_data-2.png" width="750px" />

General functions to prepare data
---------------------------------

The most important is `extract.balanced.xdata.ydata.network.foldid` that will take the data and build the data structures for the following steps.

``` r
source('functions/extract_data.R')
```

### Select data with expression levels

Get data that has all RNA-seq and calculate Pearson correlation on the genes. - When getting the correlation matrix, use the absolute values as edge weights

``` r
# set the index for patients with event and censored
#  this way it creates a balanced dataset in terms of train/test and
#  cross-validation
if (!is.logical(ydata$status)) {
  ydata$status <- ydata$status == 'DEAD'
}

event.ix <- which(ydata$status)
censo.ix <- which(!ydata$status)
#
#set.seed(1985)
#new.xdata <- xdata[abs(rnorm(nrow(xdata))) < .5,]
#flog.info('Can\' do it with %d variables, reducing randomly to %d', nrow(xdata), nrow(new.xdata))
#dim(new.xdata)
#xdata <- new.xdata
#
result <- extract.balanced.xdata.ydata.network.foldid(xdata, ydata, event.ix, censo.ix,
                                                      train.perc = training.set, nfolds = nfolds,
                                                      test.with.all = FALSE,
                                                      network.normalizationType = NULL,
                                                      xdata.normalizationType = 'colCenterScale',
                                                      centrality.normalizationType = 'max')
```

    ## INFO [2016-12-23 16:10:52 29681] Generating train and test sets...
    ## INFO [2016-12-23 16:10:52 29681] Normalize XDATA by colCenterScale
    ## INFO [2016-12-23 16:11:24 29681] Genereating co-expression network from training set...
    ## INFO [2016-12-23 16:11:27 29681] Generating xdata and ydata for test and train...
    ## INFO [2016-12-23 16:11:33 29681] Generating foldid for cross-validation...
    ## INFO [2016-12-23 16:11:33 29681] Generating centrality information...
    ## INFO [2016-12-23 16:11:33 29681] path data/degree-vector.RData

``` r
# show all names
flog.info('Assigned variables from function:', names(result), capture = T)
```

    ## INFO [2016-12-23 16:11:33 29681] Assigned variables from function:
    ## 
    ## [1] "xdata.train" "xdata.test"  "ydata.train" "ydata.test"  "network"    
    ## [6] "foldid"      "degree"

``` r
# assign all variables from the function
for (var.name in names(result))
  assign(var.name, result[[var.name]])
# don't need result anymore
rm(result)
# run garbace collector
suppressMessages({
  gc(verbose = F)
  gc(verbose = F)
  gc(reset = T, verbose = F)
})
```

    ##             used   (Mb) gc trigger   (Mb)  max used   (Mb)
    ## Ncells   3012682  160.9    4703850  251.3   3012682  160.9
    ## Vcells 260106725 1984.5  589365747 4496.6 260106725 1984.5

``` r
# convert from days to months
ydata.test$time  <- ydata.test$time / 365 * 12
ydata.train$time <- ydata.train$time / 365 * 12
#
flog.info('Test dataset has %d samples that have died and %d that are censored.', 
          sum(ydata.test$status),
          sum(!ydata.test$status))
```

    ## INFO [2016-12-23 16:11:35 29681] Test dataset has 16 samples that have died and 142 that are censored.

``` r
flog.info('Train dataset has %d samples that have died and %d that are censored.', 
          sum(ydata.train$status),
          sum(!ydata.train$status))
```

    ## INFO [2016-12-23 16:11:35 29681] Train dataset has 88 samples that have died and 801 that are censored.

``` r
#
```
