library(futile.logger)

cv.glmnet.mine <- function (x, y, weights, offset = NULL, lambda = NULL, type.measure = c("mse", 
                            "deviance", "class", "auc", "mae"), nfolds = 10, foldid,
                            grouped = TRUE, keep = FALSE, mc.cores = 1, ...) 
{
  if (missing(type.measure)) 
    type.measure = "default"
  else type.measure = match.arg(type.measure)
  if (!is.null(lambda) && length(lambda) < 2) 
    stop("Need more than one value of lambda for cv.glmnet")
  N = nrow(x)
  if (missing(weights)) 
    weights = rep(1, N)
  else weights = as.double(weights)
  y = drop(y)
  glmnet.call = match.call(expand.dots = TRUE)
  which = match(c("type.measure", "nfolds", "foldid", "grouped", 
                  "keep"), names(glmnet.call), F)
  if (any(which)) 
    glmnet.call = glmnet.call[-which]
  glmnet.call[[1]] = as.name("glmnet")
  glmnet.object = glmnet(x, y, weights = weights, offset = offset, 
                         lambda = lambda, ...)
  glmnet.object$call = glmnet.call
  is.offset = glmnet.object$offset
  if (inherits(glmnet.object, "multnet") && !glmnet.object$grouped) {
    nz = predict(glmnet.object, type = "nonzero")
    nz = sapply(nz, function(x) sapply(x, length))
    nz = ceiling(apply(nz, 1, median))
  }
  else nz = sapply(predict(glmnet.object, type = "nonzero"), 
                   length)
  if (missing(foldid)) 
    foldid = sample(rep(seq(nfolds), length = N))
  else nfolds = max(foldid)
  if (nfolds < 3) 
    stop("nfolds must be bigger than 3; nfolds=10 recommended")
  outlist = as.list(seq(nfolds))
  if (mc.cores > 1) {
    outlist = mclapply(seq(nfolds), function(i)
    {
      which = foldid == i
      if (is.matrix(y)) 
        y_sub = y[!which, ]
      else y_sub = y[!which]
      if (is.offset) 
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      glmnet(x[!which, , drop = FALSE], y_sub, lambda = lambda, 
             offset = offset_sub, weights = weights[!which], 
             ...)
    }, mc.cores = mc.cores)
    flog.info('GLMNET: Finished parallel part.')
  }
  else {
    for (i in seq(nfolds)) {
      which = foldid == i
      if (is.matrix(y)) 
        y_sub = y[!which, ]
      else y_sub = y[!which]
      if (is.offset) 
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      outlist[[i]] = glmnet(x[!which, , drop = FALSE], 
                            y_sub, lambda = lambda, offset = offset_sub, 
                            weights = weights[!which], ...)
    }
  }
  fun = paste("cv", class(glmnet.object)[[1]], sep = ".")
  flog.info('GLMNET: calling %s', fun)
  lambda = glmnet.object$lambda
  if (fun == 'cv.coxnet') {
    fun = 'cv.coxnet.mine'
    cvstuff = do.call(fun, list(outlist, lambda, x, y, weights, 
                                offset, foldid, type.measure, grouped, keep, mc.cores))
  } else {
    cvstuff = do.call(fun, list(outlist, lambda, x, y, weights, 
                                offset, foldid, type.measure, grouped, keep))
  }
  flog.info('GLMNET: got cvstuff.')
  cvm = cvstuff$cvm
  cvsd = cvstuff$cvsd
  nas = is.na(cvsd)
  if (any(nas)) {
    lambda = lambda[!nas]
    cvm = cvm[!nas]
    cvsd = cvsd[!nas]
    nz = nz[!nas]
  }
  cvname = cvstuff$name
  out = list(lambda = lambda, cvm = cvm, cvsd = cvsd, cvup = cvm + 
               cvsd, cvlo = cvm - cvsd, nzero = nz, name = cvname, 
             glmnet.fit = glmnet.object)
  if (keep) 
    out = c(out, list(fit.preval = cvstuff$fit.preval, foldid = foldid))
  lamin = if (cvname == "AUC") 
    getmin(lambda, -cvm, cvsd)
  else getmin(lambda, cvm, cvsd)
  obj = c(out, as.list(lamin))
  class(obj) = "cv.glmnet"
  obj
}


cv.coxnet.mine <- function (outlist, lambda, x, y, weights, offset, foldid, type.measure, 
                            grouped, keep = FALSE, mc.cores = 14) 
{
  typenames = c(deviance = "Partial Likelihood Deviance")
  if (type.measure == "default") 
    type.measure = "deviance"
  if (!match(type.measure, c("deviance"), FALSE)) {
    warning("Only 'deviance'  available for Cox models; changed to type.measure='deviance'")
    type.measure = "deviance"
  }
  if (!is.null(offset)) {
    is.offset = TRUE
    offset = drop(offset)
  }
  else is.offset = FALSE
  nfolds = max(foldid)
  if ((length(weights)/nfolds < 10) && !grouped) {
    warning("Option grouped=TRUE enforced for cv.coxnet, since < 3 observations per fold", 
            call. = FALSE)
    grouped = TRUE
  }
  cvraw = matrix(NA, nfolds, length(lambda))
  mlami = max(sapply(outlist, function(obj) min(obj$lambda)))
  which_lam = lambda >= mlami
  cvraw.list = mclapply(seq(nfolds), function(i) {
    which = foldid == i
    fitobj = outlist[[i]]
    coefmat = predict(fitobj, type = "coeff", s = lambda[which_lam])
    if (grouped) {
      plfull = coxnet.deviance(x = x, y = y, offset = offset, 
                               weights = weights, beta = coefmat)
      plminusk = coxnet.deviance(x = x[!which, ], y = y[!which, 
                                                        ], offset = offset[!which], weights = weights[!which], 
                                 beta = coefmat)
      cvraw[i, seq(along = plfull)] = plfull - plminusk
    }
    else {
      plk = coxnet.deviance(x = x[which, ], y = y[which, 
                                                  ], offset = offset[which], weights = weights[which], 
                            beta = coefmat)
      cvraw[i, seq(along = plk)] = plk
    }
  }, mc.cores = mc.cores)
  # consolidate cvraw in a matrix
  for (i in seq(nfolds)) {
    cvraw[i, seq(along = cvraw.list[[i]])] = cvraw.list[[i]]
  }
  
  status = y[, "status"]
  N = nfolds - apply(is.na(cvraw), 2, sum)
  weights = as.vector(tapply(weights * status, foldid, sum))
  cvraw = cvraw/weights
  cvm = apply(cvraw, 2, weighted.mean, w = weights, na.rm = TRUE)
  cvsd = sqrt(apply(scale(cvraw, cvm, FALSE)^2, 2, weighted.mean, 
                    w = weights, na.rm = TRUE)/(N - 1))
  out = list(cvm = cvm, cvsd = cvsd, name = typenames[type.measure])
  if (keep) 
    warning("keep=TRUE not implemented for coxnet")
  out
}
