source(file.path('functions', 'run_forest.R'))
source(file.path('functions', 'cv.glmnet.mine.R'))

#' Extract all information required to perform regularization
#' 
#' It defines:
#'  - Balanced training and test set
#'  - Balanced foldids for train dataset
#'  - Network of co-expression from xdata
#'  - Centrality information (currently only degree)
#'
#' @param data.xdata Matrix of xdata (could be gene expression)
#' @param data.ydata Survival data.frame with time and status
#' @param event.ix rows in ydata/xdata where event has occured
#' @param censo.ix rows in ydata/xdata where event has NOT occured
#' @param train.perc percentage of training set from all data (balanced)
#' @param nfolds number of folds in cross validation
#' @param test.with.all test dataset should have all data (train + test)
#' @param xdata.normalizationType normalization on xdata
#' @param network.normalizationType normalization on network
#' @param centrality.normalizationType normalization on centrality
#'
#' @return 
#' *xdata.train, xdata.test, ydata.train, ydata.test*
#'  - Balanced training and test set
#' *foldid*
#'  - Balanced foldids for train dataset
#' *network*
#'  - Network of co-expression from xdata
#' *degree*
#'  - Centrality information (currently only degree)
#'  
#' @export
#'
#' @examples
extract.balanced.xdata.ydata.network.foldid <- function(data.xdata, data.ydata, 
                                                        event.ix, censo.ix, 
                                                        train.perc,
                                                        nfolds = 10,
                                                        test.with.all = FALSE,
                                                        xdata.normalizationType = NULL,
                                                        network.normalizationType = NULL,
                                                        centrality.normalizationType = NULL,
                                                        training.seed = NULL) {
  #
  # get training and test dataset
  flog.info('Generating train and test sets...')
  data.separated.ix <- gen.train.test(event.ix, censo.ix, train.perc, training.seed = training.seed)
  train.ix <- data.separated.ix$train
  test.ix  <- data.separated.ix$test
  #
  flog.info('Normalize XDATA by %s', xdata.normalizationType)
  data.xdata <- normMatrix(data.xdata, xdata.normalizationType)
  #
  # calculate network
  flog.info('Genereating co-expression network from training set...')
  train.expr <- data.xdata[, train.ix]
  # Must transpose the matrix, such as each column is a feature that will calculate the correlation
  #
  # do it pairwise
  #
  if (nrow(data.xdata) > 1) {
    #cor.pearson <- alternateCor(data.xdata, 14) # 8 number of cores
    cor.pearson <- 1
  }
  else {
    cor.pearson <- cor(t(train.expr), method = 'pearson')
    # remove diagonal
    diag(cor.pearson) <- 0
  }
  # set absolute values
  cor.pearson.abs <- abs(cor.pearson)
  #
  #flog.info('Normalize network information by %s...', network.normalizationType)
  #network <- normMatrix(cor.pearson.abs, network.normalizationType, abs = TRUE)
  network <- array(0, nrow(data.xdata))
  #
  # xdata and ydata
  flog.info("Generating xdata and ydata for test and train...")
  if (test.with.all) {
    test.ix <- sort(c(train.ix, test.ix))
  }
  xdata.test <- t(data.xdata[, test.ix])
  ydata.test <- data.ydata[test.ix,]
  #
  xdata.train <- t(data.xdata[, train.ix])
  ydata.train <- data.ydata[train.ix,]
  #
  # Generate Cross-Validation Fold ID for train set
  flog.info("Generating foldid for cross-validation...")
  foldid <- gen.foldid(which(!ydata.train$status), which(ydata.train$status), nfolds)
  #
  # Generate Centrality information
  flog.info('Generating centrality information...')
  #degree <- generateDegree(network, centrality.normalizationType)
  flog.info('path %s', file.path('data', 'degree-vector.RData'))
  load(file.path('data', 'degree-vector.RData'))
  #
  #
  # Return results
  return(list(xdata.train = xdata.train, xdata.test = xdata.test,
              ydata.train = ydata.train, ydata.test = ydata.test,
              network = network, foldid = foldid, degree = degree))
}

#' Generate degree from network (matrix)
#'
#' @param network matrix representing network (assumes undirected)
#' @param normalizeType type of normalization (max, center, centerScale)
#'
#' @return vector with the degree of each node
#' @export
#'
#' @examples
#' # random network with 5 nodes
#' network <- matrix(rnorm(25), 5)
#' # generates degree by normalizing with maximum value
#' generateDegree(network, 'max')
generateDegree <- function(network, normalizeType) {
  # Simple sum of node edges (only upper triangle)
  degree <- 1 / (rowSums(triu(network, 1)) + colSums(triu(network, 1)))
  degree <- abs(degree)
  degree[is.na(degree) || is.nan(degree) || is.infinite(degree)] <- 0
  # normalize
  if (!is.null(normalizeType)) {
    degree <- normMatrix(degree, paste0('col', capitalize(normalizeType)))
    degree[is.na(degree) || is.nan(degree) || is.infinite(degree)] <- 0
  }
  #
  # show summaries
  flog.info('Summary on degree (absolute):',summary(degree),capture = TRUE)
  #
  return(degree)
}

#' Normalize matrix
#'
#' @param my.matrix matrix to be normalized
#' @param normalizationType type of normalization (zhang, {col,row}Max, {col,row}Center, {col,row}CenterScale)
#' @param abs performs absolute value of resulting normalization
#'
#' @return normalized matrix
#' @export
#'
#' @examples
#' # random network with 5 nodes
#' network <- matrix(rnorm(25), 5)
#' normMatrix(network, 'colMax')
normMatrix <- function(my.matrix, normalizationType, abs = FALSE) {
  # check if is vector and convert to matrix
  convert.to.vector <- is.vector(my.matrix)
  if (convert.to.vector)
    my.matrix <- as.matrix(my.matrix)
  if (is.null(normalizationType)) {
    out.matrix <- my.matrix
  } else if (normalizationType == 'zhang') {
    out.matrix <- normalize.zhang(my.matrix)
  } else if (normalizationType == 'colMax') {
    out.matrix <- sapply(1:ncol(my.matrix), function(col.ix){
      my.matrix[,col.ix] / max(my.matrix[,col.ix])
    })  
  } else if (normalizationType == 'rowMax') {
    out.matrix <-sapply(1:nrow(my.matrix), function(row.ix){
      my.matrix[row.ix,] / max(my.matrix[row.ix,])
    })
    out.matrix <- t(out.matrix)
  } else if (normalizationType == 'colCenter') {
    out.matrix <- scale(my.matrix, center = T, scale = F)
  } else if (normalizationType == 'rowCenter') {
    out.matrix <- t(scale(t(my.matrix), center = T, scale = F))
  } else if (normalizationType == 'colCenterScale') {
    out.matrix <- scale(my.matrix, center = T, scale = T)
  } else if (normalizationType == 'rowCenterScale') {
    out.matrix <- t(scale(t(my.matrix), center = T, scale = T))
  }
  if (abs)
    out.matrix <- abs(out.matrix)
  if (convert.to.vector)
    out.matrix <- as.vector(out.matrix)
  return(out.matrix)
}

#' Normalizes the matrix according to Zhang et al. (2014)
#'
#' @param network 
#'
#' @return normalized matrix
#' @export
#'
#' @examples
normalize.zhang <- function(network) {
  #
  # Normalize matrix
  # use the md5 digest to create a signature of the .abs to store .norm
  cache.file <- file.path('cache', 
                          paste0('cor.pearson.abs.', 
                                 digest(network, algo = 'md5'), 
                                 '.RData'))
  
  # if a cached file already exists with the same signature, then it should be used
  #  instead of recalculating
  if (file.exists(cache.file)) {
    flog.info('%s\n%s', 'Loading cache file instead of calculating all...',
              '  (note that cache file is referenced by md5sum of cor.pearson.abs matrix)')
    load(cache.file)
  } else if (FALSE) {
    flog.info('Performing the normalization of cor.pearson.abs... (this could take a while)')
    sum.cols <- sqrt(colSums(network))
    sum.rows <- sqrt(rowSums(network))
    
    # divide the problem in 4 different partitions
    my.probs <- seq(0,1, 1 / (getDoParWorkers()))
    my.probs <- my.probs[2:length(my.probs)]
    quants <- c(0,round(quantile(1:nrow(network), probs = my.probs)))
    # for each partition calculate the normalization
    network.norm <- foreach( ix = 2:length(quants), .combine = rbind ) %dopar% {
      # foreach was not very eficient done row by row, so this partion scheme
      #  must be done, probably a better parallization framework could have been
      #  used
      t(sapply((quants[ix-1] + 1):quants[ix], function(row.ix) {
        # actuall calculations done here
        sapply(1:ncol(network), function(col.ix) {
          network[row.ix,col.ix] / sum.cols[col.ix] / sum.rows[row.ix]
        })
      }))
      # seems like calculations are done pretty quick, rsession takes a while to
      #  join the results
    }
    save(file = cache.file, 'network.norm')
  }
  # flog.info('cor.pearson.norm has size of %d rows and %d columns (which should be the same)',
  #           nrow(cor.pearson.norm), ncol(cor.pearson.norm))
  return(network.norm)
}

#' Generate a balanced train and test set
#'
#' @param event.ix index of events
#' @param censo.ix index of censored events
#' @param train.perc training percentage
#'
#' @seealso verissimo::balanced.train.and.test
#' 
#' @return list with training and test set
#' @export
#'
#' @examples
gen.train.test <- function(event.ix, censo.ix, train.perc, training.seed = NULL) {
  #
  if (train.perc == 1) {
    test.ix  <- sort(c(event.ix, censo.ix))
    train.ix <- test.ix
  } else {
    if (!is.null(training.seed))
      set.seed(training.seed)
    ixs <- balanced.train.and.test(event.ix, censo.ix, train.perc = train.perc, join.all = T)
    train.ix <- ixs$train
    test.ix  <- ixs$test
    # create variable with expression data
    # TODO: filter the assay data in lpdTrain to only RNA data
  }
  return(list(test = test.ix, train = train.ix))
}

#' Generate balanced foldid for cross validation
#'
#' @param event.ix index of events
#' @param censo.ix index of censored events
#' @param nfolds number of folds
#'
#' @return vector with foldids defined
#' @export
#'
#' @examples
gen.foldid <- function(event.ix, censo.ix, nfolds = 10) {
  # generate fold ids
  foldid.result <- balanced.cv.folds(event.ix, censo.ix, nfolds = nfolds)
  # consolidate
  foldid <- array(0, length(c(event.ix, censo.ix))) 
  foldid[event.ix] <- foldid.result$output[[1]]
  foldid[censo.ix] <- foldid.result$output[[2]]
  #
  return(foldid)
}

#' Call cross-validation glmnet
#' 
#' Auxiliary method that helps clean up code
#'
#' @param xdata 
#' @param ydata 
#' @param penalty.factor either a vector of ones or of weighted penalties defined by centrality metric
#'
#' @return
#' @export
#'
#' @examples
my.cv.glmnet <- function(xdata, ydata, penalty.factor, alpha, 
                         nlambda, foldid, mc.cores = 1) {
  return(cv.glmnet(xdata, Surv(ydata[,1], ydata[,2]), 
                   foldid = foldid,
                   mc.cores = mc.cores, 
                   family='cox', 
                   alpha = alpha, nlambda = nlambda, 
                   standardize = T, 
                   penalty.factor = penalty.factor))
}

#' Show results and optionally fit a coxph model
#'
#' @param xdata xdata to be fitted
#' @param ydata ydata to be fitted
#' @param results cv.glmnet results
#' @param with.coxph fit a coxph model?
#'
#' @return the model's "best" coefficients or coxph
#' @export
#'
#' @examples
print.results <- function(xdata, ydata, results, with.coxph = F, var.num = NULL) {
  lambda.min.ix <- which(results$lambda == results$lambda.min)
  lambda.1se.ix <- which(results$lambda == results$lambda.1se)
  #
  lambda.chosen.ix <- sort(results$cvm, index.return =T)$ix[1]
  
  #
  #
  # How to choose number of variables
  #
  if (is.null(var.num)) {
    target = 'lambda.min'
  } else {
    # when looking for a target number of variables
    #  if it does not have it, then look for the largest one below
    #  below that one
    var.num.ix <- as.vector(which(results$nzero >= var.num)[1])
    if (is.na(var.num.ix)) {
      var.num.ix <- length(results$nzero)
    }
    target <- results$lambda[var.num.ix]
  }
  if (sum(coef(results, s = 'lambda.min') != 0) == 0)
    flog.warn('Best lambda has 0 variables!')
  #flog.info("best lambda: %f | non-zero: %d", results$lambda.min,
  #          sum(coef(results, s = 'lambda.min') != 0))
  #flog.info(" 1se lambda: %f | non-zero: %d", results$lambda.1se,
  #          sum(coef(results, s = 'lambda.1se') != 0))
  #flog.info(" chs lambda: %f | non-zero: %d", results$lambda[lambda.chosen.ix],
  #          sum(results$glmnet.fit$beta[,lambda.chosen.ix] != 0))
  # cox.model was not really required, just the coefficients
  if (with.coxph) {
    # reduce dimensionality
    init.vals <- coef(results, s = target)
    non.zero.ix <- which(init.vals != 0)
    init.vals <- init.vals[non.zero.ix]
    # calculate cox.model based on fitted coefficients
    #cox.model <- coxph.fit(xdata[,non.zero.ix], ydata, strata = NULL, init = init.vals, 
    #                       control = coxph.control(outer.max = 0, iter.max = 0), method = 'breslow',
    #                       rownames = NULL)
    new.xdata <- as.data.frame(xdata[,non.zero.ix])
    colnames(new.xdata) <- colnames(xdata)[non.zero.ix]
    cox.model <- coxph( Surv(time, status) ~ ., data = cbind(ydata,new.xdata), init = init.vals,
                       control = coxph.control(outer.max = 0, iter.max = 0), ties = 'breslow')
  } else {
    #
    # compatibility with coxph.fit model
    cox.model <- list(coefficients = coef(results, s = 'lambda.min'))
  }
  return(cox.model)
}

#' Proxy function to draw kaplan meier curve
#'
#' @param coeff coefficients to be used
#' @param xdata xdata to be fitted in KM
#' @param ydata xdata to be fitted in KM
#' @param name name that is shown in plots
#'
#' @return plot and km object
#' @export
#'
#' @examples
my.draw.kaplan <- function(coeff, xdata, ydata, name) {
  my.list <- list()
  my.list[[' ']] <- coeff
  var.names <- names(coeff)
  if (is.null(var.names))
    var.names <- rownames(coeff)
  draw.kaplan(my.list, xdata[,var.names], ydata, 
              legend.outside = F, filename = name, expand.yzero = T)
}

#' Proxy function to call on methods
#'
#' @param penalty penalty vector to be used
#' @param name name shown in plots
#' @param my.penalty 
#' @param with.coxph 
#'
#' @return
#' @export
#'
#' @examples
calculate.reg <- function(alpha, name, xdata.train, ydata.train, xdata.test, ydata.test, foldid,
                          my.penalty = NULL, with.coxph = T, test.with.all = F,
                          nlambda = 100, mc.cores = 1, var.num = NULL, results = NULL) {
  #
  my.penalty.was <- my.penalty
  if (is.null(my.penalty))
    my.penalty <- array(1, ncol(xdata.train))
  #
  # test with all
  my.ydata.test <- ydata.test
  my.xdata.test <- xdata.test
  if (test.with.all) {
    my.xdata.test <- rbind(my.xdata.test, xdata.train)
    my.ydata.test <- rbind(my.ydata.test, ydata.train)
  }
  #
  if (is.null(results)) {
    results   <- my.cv.glmnet(xdata.train, ydata.train, 
                              my.penalty, alpha, nlambda, 
                              foldid, mc.cores = mc.cores)
  }
  # flog.info('Building cox model... (it can take a while)')
  cox.model <- print.results(xdata.train, ydata.train, 
                             results, with.coxph = with.coxph, 
                             var.num)
  #
  # title
  my.title <- paste0(name, ' (', length(cox.model$coefficients), ' co-variates)')
  if (!is.null(my.penalty.was)) {
    my.title <- paste0(my.title, ' (degree)')
  }
  km <- my.draw.kaplan(cox.model$coefficients, 
                       my.xdata.test, my.ydata.test, my.title
                       )
  var.names <- names(cox.model$coefficients)
  if (is.null(var.names))
    var.names <- rownames(cox.model$coefficients)
  return(list(results = results, cox.model = cox.model, km = km, var.names = var.names))
}


